<?php

// Reminder: always indent with 4 spaces (no tabs).
// +---------------------------------------------------------------------------+
// | GoogleAnalytics Plugin 1.1.1 for Geeklog - The Ultimate Weblog            |
// +---------------------------------------------------------------------------|
// | plugins/googleanalytics/functions.inc                                     |
// +---------------------------------------------------------------------------|
// | Copyright (C) 2009-2010 by hiroron - hiroron AT hiroron DOT com           |
// |                                                                           |
// | Constructed with the Universal Plugin                                     |
// | Copyright (C) 2002 by the following authors:                              |
// | Tom Willett               -    twillett AT users DOT sourceforge DOT net  |
// | Blaine Lang               -    langmail AT sympatico DOT ca               |
// | The Universal Plugin is based on prior work by:                           |
// | Tony Bibbs                -    tony AT tonybibbs DOT com                  |
// | Modified by:                                                              |
// | mystral-kk                -    geeklog AT mystral-kk DOT net              |
// | dengen                    -    taharaxp AT gmail DOT com                  |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

if (strpos(strtolower($_SERVER['PHP_SELF']), 'functions.inc') !== false) {
    die('This file can not be used on its own.');
}

/**
* Include language file
*/
$plugin_path = $_CONF['path'] . 'plugins/googleanalytics/';
$langfile = $plugin_path . 'language/' . $_CONF['language'] . '.php';

if (file_exists($langfile)) {
    require_once $langfile;
} else {
    require_once $plugin_path . 'language/japanese_utf-8.php';
}

/*
* Check and see if we need to load the plugin configuration
*/
if (!isset($_GANA_CONF['googleanalyticsloginrequired'])) {
    require_once $_CONF['path_system'] . 'classes/config.class.php';

    $plg_config = config::get_instance();
    $_GANA_CONF = $plg_config->get_config('googleanalytics');
}
$_GANA_CONF['version'] = '1.1.1';

// +---------------------------------------------------------------------------+
// | Geeklog Plugin API Implementation                                         |
// +---------------------------------------------------------------------------+

/**
* Called by the plugin Editor to display the current plugin code version
* This may be different than the version installed and registered currently.
* If newer then you may want to run the update
*
* @return   string  version number
*
*/
function plugin_chkVersion_googleanalytics()
{
    global $_GANA_CONF;

    return $_GANA_CONF['version'];
}

/**
* Upgrade the googleanalytics plugin
*
* @return   int     Number of message to display (true = generic success msg)
*
*/
function plugin_upgrade_googleanalytics()
{
    global $_CONF, $_TABLES, $_GANA_CONF;

    // the plugin needs these function so complain when they don't exist
    if (!function_exists('PLG_uninstall') ||
            !function_exists('COM_createLink')) {
        return 3002;
    }

    $pi_name = 'googleanalytics';
    $currentVersion = DB_getItem($_TABLES['plugins'],'pi_version',"pi_name='{$pi_name}'");

    // Do upgrading
    switch ($currentVersion) {
        case '1.0.0':
            $c = config::get_instance();
            $c->add('asynchronous', 1, 'select', 0, 0, 0, 30, true, $pi_name);
            $c->add('nomobilebottracking', 1, 'select', 0, 0, 0, 40, true, $pi_name);
            break;
        case '1.1.0':
            $_GANA_DEFAULT['mobilebotua'] = array('Googlebot-Mobile', 'BaiduMobaider', 'Y!J-SRD', 'Y!J-MRD', 'moba-crawler', 'mobile goo', 'LD_mobile_bot', 'froute.jp');
            $c = config::get_instance();
            $c->add('fs_mobilebotua', NULL, 'fieldset', 0, 1, NULL, 0, true, $pi_name);
            $c->add('mobilebotua', $_GANA_DEFAULT['mobilebotua'], '%text', 0, 1, NULL, 110, true, $pi_name);
            break;
        default:
            break;
    }

    // Update plugin info
    DB_query ("UPDATE {$_TABLES['plugins']} "
            . "SET pi_version = '{$_GANA_CONF['version']}', pi_gl_version = '" . VERSION . "' "
            . "WHERE pi_name = '{$pi_name}'");
    COM_errorLog($pi_name . " plugin was successfully updated to version {$_GANA_CONF['version']}.");

    return true;
}

/**
* Automatic uninstall function for plugins
*
* @return   array
*
* This code is automatically uninstalling the plugin.
* It passes an array to the core code function that removes
* tables, groups, features and php blocks from the tables.
* Additionally, this code can perform special actions that cannot be
* foreseen by the core code (interactions with other plugins for example)
*
*/
function plugin_autouninstall_googleanalytics()
{
    $out = array(
        // give the name of the tables, without $_TABLES[]
        'tables' => array('googleanalytics'),

        // give the full name of the group, as in the db
        'groups' => array('GoogleAnalytics Admin'),

        // give the full name of the feature, as in the db
        'features' => array('googleanalytics.admin'),

        //// give the full name of the block, including 'phpblock_', etc
        //'php_blocks' => array('phpblock_googleanalytics'),

        // give all vars with their name
        'vars'=> array()
    );
    return $out;
}

function plugin_templatesetvars_googleanalytics ($templatename, &$template)
{
    global $_GANA_CONF, $_CONF;
    
    switch ($templatename) {
        case 'header':
        case 'footer':
            if (isset($_GANA_CONF['gatrackingid']) && $_GANA_CONF['gatrackingid'] != 'UA-XXXXXXX-X') {
                $ua_id = $_GANA_CONF['gatrackingid'];
                if ($_CONF['theme'] == 'mobile_3g' || $_CONF['theme'] == 'mobile') {
                    // mobile
                    if (!empty($_GANA_CONF['gatrackingid_mobile']) && $_GANA_CONF['gatrackingid_mobile'] != 'UA-XXXXXXX-X' ) {
                        $ua_id = $_GANA_CONF['gatrackingid_mobile'];
                    }
                    if ( !googleanalytics_isbot_mobile() ) {    // bot check
                        $ret  = '<!-- mobile GAnalytics .' . $ua_id;
                        $ret .= ' sendReqCode(' . googleanalytics_mobile_code_sending($ua_id) . ')';
                        $ret .= ' -->';
                    }

                } else {
                    // pc : Synchronous Tracking
                    if (! isset($_GANA_CONF['asynchronous']) || ! $_GANA_CONF['asynchronous']) {
                        $ret = <<<EOF
<script type="text/javascript">
var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");
document.write(unescape("%3Cscript src='" + gaJsHost + "google-analytics.com/ga.js' type='text/javascript'%3E%3C/script%3E"));
</script>
<script type="text/javascript">
try {
var pageTracker = _gat._getTracker("{$ua_id}");
pageTracker._trackPageview();
} catch(err) {}</script>
EOF;
                    }
                }
                $template->set_var('googleanalytics', $ret);
            }
    }
}

function plugin_getheadercode_googleanalytics ()
{
    global $_GANA_CONF, $_CONF;
    
    $ret = null;
    if (isset($_GANA_CONF['gatrackingid']) && $_GANA_CONF['gatrackingid'] != 'UA-XXXXXXX-X') {
        $ua_id = $_GANA_CONF['gatrackingid'];
        if ($_CONF['theme'] != 'mobile_3g' && $_CONF['theme'] != 'mobile') {
            // pc : Asynchronous Tracking
            if (isset($_GANA_CONF['asynchronous']) && $_GANA_CONF['asynchronous']) {
                $ret = <<<EOF
<script type="text/javascript">
var _gaq = _gaq || [];
_gaq.push(['_setAccount', '{$ua_id}']);
_gaq.push(['_trackPageview']);
(function() {
var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
(document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(ga);
})();
</script>
EOF;
            }
        }
    }
    return $ret;
}

function googleanalytics_mobile_code_sending ($id)
{
    global $_CONF;

    require_once 'HTTP/Request.php';

    $var_utmac   = $id;
    $var_utmhn   = str_replace('https://', '', $_CONF['site_url']);
    $var_utmhn   = str_replace('http://', '', $var_utmhn);    
    $var_utmn    = rand(1000000000,9999999999);
    $var_cookie  = '';
    $var_today   = time();
    $var_referer = $_SERVER['HTTP_REFERER'];
    $var_uservar = '';
    $var_utmp    = $_SERVER['REQUEST_URI'];
    
    $g_url = 'http://www.google-analytics.com/__utm.gif?utmwv=1&utmn='.$var_utmn.'&utmsr=-&utmsc=-&utmul=-&utmje=0&utmfl=-&utmdt=-&utmhn='.$var_utmhn.'&utmr='.$var_referer.'&utmp='.$var_utmp.'&utmac='.$var_utmac.'&utmcc=__utma%3D'.$var_cookie.'.'.$var_random.'.'.$var_today.'.'.$var_today.'.'.$var_today.'.2%3B%2B__utmb%3D'.$var_cookie.'%3B%2B__utmc%3D'.$var_cookie.'%3B%2B__utmz%3D'.$var_cookie.'.'.$var_today.'.2.2.utmccn%3D(direct)%7Cutmcsr%3D(direct)%7Cutmcmd%3D(none)%3B%2B__utmv%3D'.$var_cookie.'.'.$var_uservar.'%3B';
    
    $header = array();
    //Set the language to that of the client so analytics can track it.
    if (!empty($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
        $header['Accept-language'] = $_SERVER['HTTP_ACCEPT_LANGUAGE'].'\r\n';
    }
    //Set the user agent to that of the client so analytics can track it.
    if (!empty($_SERVER['HTTP_USER_AGENT'])) {
        $header['User-Agent'] = $_SERVER['HTTP_USER_AGENT'].'\r\n';
    }
    
    $req = new HTTP_Request($g_url);
    //$req->setMethod(HTTP_REQUEST_METHOD_HEAD);
    if (count($header) > 0) {
        foreach ( $header as $key => $val ) {
            $req->addHeader($key, $val);
        }
    }
    $response = $req->sendRequest ();
    if (PEAR::isError ($response)) {
        $msg = $response->getMessage();
        COM_errorLog('GANA: mobile code sending error! - ' . $msg);
        return 999;
    } else {
        // all status coce http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
        return $req->getResponseCode();
    }
}

function googleanalytics_isbot_mobile ()
{
    global $_GANA_CONF;
    
    if (! isset($_GANA_CONF['nomobilebottracking']) || ! $_GANA_CONF['nomobilebottracking']) {
        return false;
    }
    
    $bots = array(
        'Googlebot-Mobile',
        'BaiduMobaider',
        'Y!J-SRD',
        'Y!J-MRD',
        'moba-crawler',
        'mobile goo',
        'LD_mobile_bot',
        'froute.jp'
    );
    if ( isset($_GANA_CONF['mobilebotua']) && $_GANA_CONF['mobilebotua'] ) {
        $bots = $_GANA_CONF['mobilebotua'];
    }
    
    foreach ($bots as $bot) {
        if (!empty($_SERVER['HTTP_USER_AGENT']) && strpos($_SERVER['HTTP_USER_AGENT'], $bot) !== false) {
            return true;
        }
    }
    return false;
}

?>